@@ -282,7 +282,7 @@ class DWConfig(DWBaseConfig):
             section_name = 'LandsatMasks'
         elif product == 'S2_THEIA':
             section_name = 'TheiaMasks'
-        elif product == 'S2_S2COR':
+        elif product in ['S2_S2COR', 'S2_PLANETARY']:
             section_name = 'S2CORMasks'
         else:
             section_name = None
@@ -302,23 +302,94 @@ class DWutils:
               }
 
     @staticmethod
-    def parse_img_name(name, img_type='S2_S2COR'):
+    def parse_sat_name(folder, img_type='S2_THEIA'):
+        img = Path(folder)
+        if img_type == 'S2_THEIA':
+            return DWutils.parse_maja_name(img.stem)
+        elif img_type == 'S2_S2COR':
+            return DWutils.parse_s2cor_name(img.stem)
+        elif img_type == 'S2_PLANETARY':
+            return DWutils.parse_planetary_name(img.stem)
+        else:
+            raise Exception('Only MAJA type is supported')
+
+    @staticmethod
+    def parse_planetary_name(name: str):
+        """
+        Get the string with a MS PLANETARY name and extract the useful information from it.
+        @param name: Image name
+        @return: Dictionary with the values.
+        """
 
         # ignore extension
         name = name.split('.')[0]
 
-        if img_type == 'S2_S2COR':
-            lst = name.split('_')
-            return dict(mission=lst[0],
-                        level=lst[1],
-                        datetime=lst[2],
-                        pdgs=lst[3],
-                        orbit=lst[4],
-                        tile=lst[5])
+        lst = name.split('_')
 
-        else:
-            print(f'Image type {img_type} is not supported')
-            return None
+        result = dict(mission=lst[0],
+                      level=lst[1][3:],
+                      datetime=lst[2],
+                      orbit=lst[3],
+                      tile=lst[4][1:])
+
+        result['date'], result['time'] = result['datetime'].split('T')
+        result['year'] = result['date'][:4]
+        result['month'] = result['date'][4:6]
+        result['day'] = result['date'][6:]
+
+        return result
+
+    @staticmethod
+    def parse_maja_name(name: str):
+        """
+        Get the string with a MAJA img name (THEIA format) and extract the useful information from it.
+        @param name: Image name
+        @return: Dictionary with the values.
+        """
+
+        # ignore extension
+        name = name.split('.')[0]
+
+        mission, datetime, level, tile, metadata, version = name.split('_')
+        date, time, _ = datetime.split('-')
+
+        result = dict(mission='S' + mission[-2:],
+                      level=level,
+                      datetime=f'{date}T{time}',
+                      tile=tile[1:],
+                      date=date,
+                      time=time,
+                      year=date[:4],
+                      month=date[4:6],
+                      day=date[6:]
+                      )
+        return result
+
+    @staticmethod
+    def parse_s2cor_name(name: str):
+        """
+        Get the string with a S2COR name (.SAFE format) and extract the useful information from it.
+        @param name: Image name
+        @return: Dictionary with the values.
+        """
+        # ignore extension
+        name = name.split('.')[0]
+
+        lst = name.split('_')
+
+        result = dict(mission=lst[0],
+                      level=lst[1][3:],
+                      datetime=lst[2],
+                      pdgs=lst[3],
+                      orbit=lst[4],
+                      tile=lst[5][1:])
+
+        result['date'], result['time'] = result['datetime'].split('T')
+        result['year'] = result['date'][:4]
+        result['month'] = result['date'][4:6]
+        result['day'] = result['date'][6:]
+
+        return result
 
     @staticmethod
     def bitwise_or(array, bit_values):
@@ -841,7 +912,7 @@ class DWutils:
         return
 
     @staticmethod
-    def create_composite(bands, folder_name, pdf=True, resolution=600):
+    def create_composite(bands, folder_name, pdf=True, resolution=600, offset=0):
 
         # copy the RGB clipped bands to output directory
 
@@ -854,8 +925,11 @@ class DWutils:
         os.system('gdalbuildvrt -separate ' + composite_base_name + '.vrt ' +
                   red_band + ' ' + green_band + ' ' + blue_band)
 
+        v_min = 0 - offset
+        v_max = 2000 - offset
+
         if pdf:
-            cmd = f'gdal_translate -of pdf -ot Byte -scale 0 2000 -outsize {resolution} 0 ' \
+            cmd = f'gdal_translate -of pdf -ot Byte -scale {v_min} {v_max} -outsize {resolution} 0 ' \
                   + composite_base_name + '.vrt ' + composite_base_name + '.pdf'
             os.system(cmd)
 
@@ -871,7 +945,7 @@ class DWutils:
         return bands_dict
 
     @staticmethod
-    def create_colorbar_pdf(product_name, title, label, colormap, min_value, max_value):
+    def create_colorbar_pdf(product_name, title, label, colormap, min_value, max_value, log_scale=False):
         # Make a figure and axes with dimensions as desired.
         fig = plt.figure(figsize=(4, 1))
         ax1 = fig.add_axes([0.05, 0.50, 0.90, 0.15])
@@ -879,8 +953,11 @@ class DWutils:
         # Set the colormap and norm to correspond to the data for which
         # the colorbar will be used.
 
-        #norm = matplotlib.colors.Normalize(vmin=min_value, vmax=max_value)
-        norm = matplotlib.colors.LogNorm(vmin=min_value, vmax=max_value)
+        if log_scale:
+            norm = matplotlib.colors.LogNorm(vmin=min_value, vmax=max_value)
+        else:
+            norm = matplotlib.colors.Normalize(vmin=min_value, vmax=max_value)
+
         #norm = matplotlib.colors.SymLogNorm(linthresh=0.03, linscale=0.03,vmin=min_value, vmax=max_value)
 
         #
