@@ -5,26 +5,32 @@ from pathlib import Path
 import re
 from waterdetect.Common import DWutils
 from waterdetect import gdal, DWProducts
+from lxml import etree
 
 
 class DWLoader:
     satellite_Dict = {
         'S2_THEIA': {'bands_names': {'Blue': 'B2', 'Green': 'B3', 'Red': 'B4', 'Mir': 'B11', 'Mir2': 'B12',
                                      'Nir': 'B8', 'Nir2': 'B8A', 'RedEdg1': 'B5', 'RedEdg2': 'B6', 'RedEdg3': 'B7'},
-                     'suffix': '.tif', 'string': 'SRE', 'metadata': '*MTD_ALL.xml', 'recursive': False},
+                     'suffix': '.tif', 'string': 'SRE', 'granule_metadata': '*MTD_ALL.xml', 'recursive': False},
+
+        'S2_PLANETARY': {'bands_names': {'Blue': 'B02_10m', 'Green': 'B03_10m', 'Red': 'B04_10m', 'Mir': 'B11_20m',
+                                         'Mir2': 'B12_20m', 'Nir': 'B08_10m', 'Nir2': 'B8A_20m', 'RedEdg1': 'B05_20m',
+                                         'RedEdg2': 'B06_20m', 'RedEdg3': 'B07_20m'},
+                         'suffix': '.tif', 'string': '', 'granule_metadata': '*MTD_TL.xml', 'recursive': False},
 
         'S2_S2COR': {'bands_names': {'Blue': 'B02_10m', 'Green': 'B03_10m', 'Red': 'B04_10m', 'Mir': 'B11_20m',
                                      'Mir2': 'B12_20m', 'RedEdg1': 'B05_20m', 'RedEdg2': 'B06_20m',
                                      'RedEdg3': 'B07_20m', 'Nir': 'B08_10m', 'Nir2': 'B8A_20m'},
 
-                     'suffix': '.jp2', 'string': '', 'metadata': '*MTD_TL.xml',
+                     'suffix': '.jp2', 'string': '', 'granule_metadata': '*MTD_TL.xml', 'metadata': 'MTD_MSIL2A.xml',
                      'subdir': 'GRANULE/*/IMG_DATA', 'recursive': True},
 
         'L8_USGS': {'bands_names': {'Green': 'B3', 'Red': 'B4', 'Mir': 'B6', 'Nir': 'B5'}},
 
         'S2_L1C': {'bands_names': {'Blue': 'B02', 'Green': 'B03', 'Red': 'B04', 'Mir': 'B11', 'Mir2': 'B12',
                                    'Nir': 'B08', 'Nir2': 'B8A', 'RedEdg1': 'B05', 'RedEdg2': 'B06', 'RedEdg3': 'B07'},
-                   'suffix': '.jp2', 'string': '', 'metadata': '*MTD_TL.xml',
+                   'suffix': '.jp2', 'string': '', 'granule_metadata': '*MTD_TL.xml',
                    'subdir': 'GRANULE/*/IMG_DATA', 'recursive': False},
 
         'L8_L1C': {'bands_names': {'Aero': 'band1', 'Blue': 'band2', 'Green': 'band3', 'Red': 'band4',
@@ -32,6 +38,9 @@ class DWLoader:
                    'suffix': '.tif', 'string': 'sr_band', 'recursive': False}
     }
 
+    band_ids = {'Blue': 1, 'Green': 2, 'Red': 3, 'RedEdg1': 4, 'RedEdg2': 5, 'RedEdg3': 6, 'Nir': 7, 'Nir2': 8,
+                'Mir': 11, 'Mir2': 12}
+
     def __init__(self, input_folder, shape_file=None, product='S2_THEIA', ref_band='Red', single_mode=False):
 
         # save the input folder (holds all the images) and the shapefile
@@ -150,10 +159,18 @@ class DWLoader:
                     bands_path /= subdir
                 else:
                     # add the first directory (hopefully the only one) to the path
-                    bands_path /= [d for d in bands_path.iterdir() if d.is_dir()][0]
+                    bands_path /= [d.name for d in bands_path.iterdir() if d.is_dir()][0]
 
         return bands_path
 
+    @property
+    def granule_metadata(self):
+        """
+        Returns the full path folder of the image's granule metadata
+        :return: Posixpath of current image's granule metadata
+        """
+        return next(self.current_image_folder.rglob(self.product_dict['granule_metadata']))
+
     @property
     def metadata(self):
         """
@@ -162,6 +179,7 @@ class DWLoader:
         """
         return next(self.current_image_folder.rglob(self.product_dict['metadata']))
 
+
         # if self.product == 'S2_THEIA':
         #     xml = Path(self.images[self._index]).rglob("*MTD_ALL.xml")
         #     for x in xml:
@@ -300,6 +318,33 @@ class DWLoader:
 
         return
 
+    def get_offset(self, band):
+        """
+        Get the offset value to be applied to the band. This offset exists in the Sen2Cor images after 25 January 2022.
+        https://sentinels.copernicus.eu/documents/247904/4830984/OMPC.CS.DQR.002.07-2022%20-%20i52r0%20-%20MSI%20L2A%20DQR%20August%202022.pdf/36edbb04-0c6c-fba3-5c34-0ba3be82e91c
+        @param band: Reflectance band
+        @return: Value to be added to the loaded band
+        """
+        if self.product == 'S2_S2COR':
+            # Open the metadata XML file and get the root
+            parser = etree.XMLParser()
+            root = etree.parse(self.metadata.as_posix(), parser).getroot()
+
+            # get the number of the band
+            band_id = self.band_ids[band]
+
+            # Search for the node in the XML tree
+            node_list = root.xpath(f".//BOA_ADD_OFFSET[@band_id={band_id}]")
+
+            # check for the results
+            if len(node_list) == 0:
+                return 0
+            else:
+                return int(node_list[0].text)
+
+        else:
+            return 0
+
     def load_raster_bands(self, bands_list: list):
 
         if len(self.gdal_bands) == 0:
@@ -313,10 +358,15 @@ class DWLoader:
 
                 raster_band = gdal_img.ReadAsArray(buf_xsize=self.x_size,
                                                    buf_ysize=self.y_size,
-                                                   resample_alg=gdal.GRA_Average).astype(dtype=np.float32) / 10000
-                self.raster_bands.update({band: raster_band})
+                                                   resample_alg=gdal.GRA_Average).astype(dtype=np.float32)
 
-                self.invalid_mask |= raster_band == -0.9999
+                self.invalid_mask |= raster_band == -9999
+
+                raster_band[raster_band != -9999] += self.get_offset(band)
+
+                raster_band /= 10000
+
+                self.raster_bands.update({band: raster_band})
 
         return self.raster_bands
 
@@ -336,12 +386,14 @@ class DWLoader:
             mask_processor = DWLandsatMaskProcessor(self.current_image_folder, self.x_size, self.y_size,
                                                     self.shape_file, self.temp_dir)
 
-        elif self.product == 'S2_S2COR':
+        elif self.product in ['S2_S2COR', 'S2_PLANETARY']:
             mask_processor = DWS2CORMaskProcessor(self.current_image_folder, self.x_size, self.y_size,
                                                   self.shape_file, self.temp_dir)
 
         if mask_processor:
             self.update_mask(mask_processor.get_combined_masks(product_masks_list))
+        else:
+            self.update_mask(np.zeros((self.y_size, self.x_size)).astype('bool'))
 
         if external_mask:
             mask_file = DWutils.find_file_glob(mask_name, self.current_image_folder)
@@ -573,7 +625,7 @@ class DWS2CORMaskProcessor:
 
     def open_gdal_masks(self, shape_file, temp_dir):
 
-        mask_file = [file for file in self.masks_folder.rglob('*SCL_20m.jp2')][0]
+        mask_file = [file for file in self.masks_folder.rglob('*SCL_20m*')][0]
         gdal_mask = gdal.Open(mask_file.as_posix())
 
         if shape_file:
@@ -659,6 +711,7 @@ class DWLandsatMaskProcessor:
 
         return combined_mask
 
+
 class DWTheiaMaskProcessor:
 
     TheiaMaskDict = {'CLM': '*_CLM_R1.tif',
